import responses
import json
from blitzcrank import Blitzcrank


class TestSummonerEndpoint:

    def test_helper_functions(self):
        summoner_name = "fakefakefake"
        api_rsp = {'accountId': '123', 'summonerLevel': 23, 'id': '234'}
        region = "euw1"
        secret = "foo"

        b = Blitzcrank(secret, region)

        with responses.RequestsMock() as rsps:
            rsps.add(responses.GET,
                     f'https://{region}.api.riotgames.com/lol/summoner/v4/summoners/by-name/{summoner_name}',
                     body=json.dumps(api_rsp))

            actual_account_id = b.summoner.get_account_id(summoner_name)
            actual_level = b.summoner.get_level(summoner_name)
            actual_summoner_id = b.summoner.get_summoner_id(summoner_name)

            assert actual_account_id == api_rsp['accountId']
            assert actual_level == api_rsp['summonerLevel']
            assert actual_summoner_id == api_rsp['id']
            assert len(rsps.calls) == 3
            assert rsps.calls[0].request.headers['X-Riot-Token'] == secret



from blitzcrank.apis.match import Match
from blitzcrank.apis.summoner import Summoner
from blitzcrank.apis.league import League
from blitzcrank.apis.champion_mastery import ChampionMastery
from blitzcrank.apis.champion import Champion
from blitzcrank.apis.item import Item
from blitzcrank.util.session import Session


class Blitzcrank:

    def __init__(self, secret, region):
        self.secret = secret
        self.region = region
        self.extended_region = self._configure_regions(region)

        self.secret = secret
        self.session = Session(secret)

        self._reload_endpoints()
        self.grab = self.Grab()

    def change_rate_limits(self, short_call_limit, short_call_period, long_call_limit, long_call_period):
        self.session = Session(self.secret, short_call_limit, short_call_period, long_call_limit, long_call_period)
        self._reload_endpoints()

    def ignore_rate_limiting(self):
        self.session = Session(self.secret, None, None, None, None)
        self._reload_endpoints()

    def _reload_endpoints(self):
        self.summoner = Summoner(self.region, self.session)
        self.match = Match(self.region, self.session)
        self.league = League(self.region, self.session)
        self.champion_mastery = ChampionMastery(self.region, self.session)
        self.champion = Champion(self.region, self.session)
        self.item = Item(self.region, self.session)

    def _configure_regions(self, region):
        if region in ["euw1", "eun1", "tr1", "ru"]:
            return "europe"
        elif region in ["na1", "br1", "la1", "la2", "oc1"]:
            return "americas"
        elif region in ["kr", "jp1"]:
            return "asia"

    # Grab will contain functions which grab data with limited context, useful for large scale analysis
    # e.g. games on a certain champion, matches in a certain elo, players of a certain elo
    class Grab:
        def games(self, tier=None, champion=None, amount=10):
            pass

        def players(self, tier=None, amount=10):
            pass



class Summoner:

    def __init__(self, region, session):
        self.region = region
        self.session = session

    # ----------------------- Base API functionality ----------------------- 

    def by_account(self, encrypted_account_id: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/summoner/v4/summoners/by-account/{encrypted_account_id}'
        ).json()

    def by_name(self, summoner_name: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/summoner/v4/summoners/by-name/{summoner_name}'
        ).json()

    def by_puuid(self, encrypted_puuid: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/summoner/v4/summoners/by-puuid/{encrypted_puuid}'
        ).json()

    def by_summoner_id(self, encrypted_summoner_id: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/summoner/v4/summoners/{encrypted_summoner_id}'
        ).json()

    # ----------------------- Helper functions ----------------------- 

    def get_account_id(self, summoner_name: str) -> str:
        return self.by_name(summoner_name)['accountId']

    def get_level(self, summoner_name: str) -> float:
        return self.by_name(summoner_name)['summonerLevel']

    def get_summoner_id(self, summoner_name: str) -> str:
        return self.by_name(summoner_name)['id']



from blitzcrank import Blitzcrank
import responses
import json


class TestRateLimiting:

    def test_default_rate_limits(self):
        secret = "fakefakefake"
        region = "euw1"
        summoner_name = "foo"
        api_rsp = {'accountId': '123'}

        b = Blitzcrank(secret, region)

        with responses.RequestsMock() as rsps:
            rsps.add(responses.GET,
                     f'https://{region}.api.riotgames.com/lol/summoner/v4/summoners/by-name/{summoner_name}',
                     body=json.dumps(api_rsp))

            for c in range(100):
                b.summoner.get_account_id(summoner_name)

            assert len(rsps.calls) == 100   # No failure even though we have exceeded the default rate limits



class League:
    def __init__(self, region, session):
        self.region = region
        self.session = session

    # ----------------------- Base API functionality -----------------------
    def by_summoner_id(self, encrypted_summoner_id: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/league/v4/entries/by-summoner/{encrypted_summoner_id}'
        ).json()

    def by_id(self, league_id: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/league/v4/leagues/{league_id}'
        ).json()

    def challenger_leagues_by_queue(self, queue: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/league/v4/challengerleagues/by-queue/{queue}'
        ).json()

    def grandmaster_leagues_by_queue(self, queue: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/league/v4/grandmasterleagues/by-queue/{queue}'
        ).json()

    def master_leagues_by_queue(self, queue: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/league/v4/masterleagues/by-queue/{queue}'
        ).json()

        # ----------------------- Helper functions -----------------------

    def rank_by_summoner_id(self, encrypted_summoner_id: str) -> dict:
        d = self.by_summoner_id(encrypted_summoner_id)
        if d:
            rank_info = {
                "tier": d['tier'],
                "rank": d['rank'],
                "lp": d['leaguePoints']
            }
        else:
            rank_info = {
                "tier": "UNRANKED",
                "rank": "IV",
                "lp": 0
            }
        return rank_info



from ratelimit import limits, sleep_and_retry
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry


class Session:

    def __init__(self, secret, short_call_limit=20, short_call_period=1, long_call_limit=100, long_call_period=120):
        self.short_call_limit = short_call_limit
        self.short_call_period = short_call_period
        self.long_call_limit = long_call_limit
        self.long_call_period = long_call_period

        retry_strategy = Retry(
            total=3,
            status_forcelist=[429, 500, 502, 503, 504],
            backoff_factor=10
        )

        self.http = requests.Session()
        self.http.mount("https://", HTTPAdapter(max_retries=retry_strategy))
        self.http.mount("http://", HTTPAdapter(max_retries=retry_strategy))

        self.headers = {'X-Riot-Token': secret}

        @sleep_and_retry
        @limits(calls=self.short_call_limit, period=self.short_call_period)
        @sleep_and_retry
        @limits(calls=self.long_call_limit, period=self.long_call_period)
        def throttler():
            return

        self.throttler = throttler

    def get(self, url, params=None, **kwargs):
        self.throttler()
        response = self.http.get(url, params=params, headers=self.headers, **kwargs)
        return response



class ChampionMastery:
    def __init__(self, region, session):
        self.region = region
        self.session = session

    def by_summoner_id(self, encrypted_summoner_id: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/champion-mastery/v4/champion-masteries/by-summoner/{encrypted_summoner_id}'
        ).json()

    def by_summoner_id_by_champ_id(self, encrypted_summoner_id: str, champion_id: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/champion-mastery/v4/champion-masteries/by-summoner/{encrypted_summoner_id}/by-champion/{champion_id}'
        ).json()

    def scores_by_summoner_id(self, encrypted_summoner_id: str) -> dict:
        return self.session.get(
            f'https://{self.region}.api.riotgames.com/lol/champion-mastery/v4/scores/by-summoner/{encrypted_summoner_id}'
        ).json()



import requests


class Champion:
    def __init__(self, region: str, session: requests.Session):
        self.region = region
        self.session = session
        self.version = self._request_dd_version()
        self.champion_data = self._request_dd_champion_data()

    def _request_dd_version(self) -> dict:
        return self.session.get("https://ddragon.leagueoflegends.com/api/versions.json").json()[0]

    def _request_dd_champion_data(self) -> dict:
        return self.session.get(f"http://ddragon.leagueoflegends.com/cdn/{self.version}/data/en_US/champion.json")\
                .json()['data']

    def by_id(self, champion_id: str) -> dict:
        return [info for name, info in self.champion_data.items() if info['key'] == str(champion_id)][0]

    def by_name(self, champion_name: str) -> dict:
        return self.champion_data[champion_name]

    def get_all(self) -> list:
        return [name for name in self.champion_data.items()]



from setuptools import setup, find_packages

from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='blitzcrank',
    packages=find_packages(),
    version='0.0.3',
    license='MIT',
    description='The Riot API in Python, made easy.',
    author='Samuel Hine',
    author_email='sam.hine27@gmail.com',
    url='https://github.com/samhine/blitzcrank',
    download_url='https://github.com/samhine/blitzcrank/archive/refs/tags/0.0.3.tar.gz',
    keywords=['PYTHON', 'RIOT', 'API'],
    install_requires=[
        'responses',
        'requests',
        'roleml',
        'pandas',
        'ratelimit'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
    ],
    long_description_content_type='text/markdown',
    long_description=long_description,
)



from datetime import datetime
from .champion import Champion
import roleml


class Match:

    def __init__(self, region, extended_region, session):
        self.region = region
        self.extended_region = extended_region
        self.champion = Champion(region, session)
        self.session = session

    # ----------------------- Base API functionality -----------------------

    def by_id(self, match_id: str) -> dict:
        return self.session.get(
            f'https://{self.extended_region}.api.riotgames.com/lol/match/v5/matches/{match_id}'
        ).json()

    def matchlist_by_account(self, puuid: str, queue: int = 420, type_: str = "", end_time: int = "",
                             begin_time: int = "", start_index: str = "", count: int = 100) -> dict:
        query_params = {
            "endTime": end_time,
            "beginTime": begin_time,
            "start": start_index,
            "count": count,
            "queue": queue,
            "type": type_,
        }
        return self.session.get(
            f'https://{self.extended_region}.api.riotgames.com/lol/match/v5/matches/by-puuid/{puuid}/ids?'
            + ''.join([k + "=" + str(v) + "&" for k, v in query_params.items() if v])
        ).json()

    def timeline_by_id(self, match_id: str) -> str:
        return self.session.get(
            f'https://{self.extended_region}.api.riotgames.com/lol/match/v5/matches/{match_id}/timeline'
        ).json()

    # ----------------------- Helper functions -----------------------

    def is_champ_in_game(self, champion_id, match_data):
        for participant in match_data['info']['participants']:
            if str(participant['championId']) == str(champion_id):
                return True
        return False

    def get_combined_game_info(self, match_id):
        return {
            "match": self.by_id(match_id),
            "timeline": self.timeline_by_id(match_id)
        }

    # NB: Will not work on custom game data since identities of participants is hidden
    def puuid_for_participant(self, participant_id, match_data):
        account_id = [player["player"]['currentAccountId'] for player in match_data["participants"] if
                      player["participantId"] == participant_id][0]
        return account_id

    # NB: Will not work on custom game data since identities of participants is hidden
    def participant_id_for_summoner(self, puuid, match_data):
        part_id = [player['participantId'] for player in match_data["participants"] if
                   player["puuid"] == puuid][0]
        return part_id

    def summoner_for_participant(self, participant_id, match_data):
        summoner_name = [player["summonerName"] for player in match_data["participants"] if
                         player["participantId"] == participant_id][0]
        return summoner_name

    def side_for_participant(self, participant_id, match_data):
        side = [info for info in match_data["participants"] if info['participantId'] == participant_id][0]['teamId']
        return "blue" if side == 100 else "red"

    def games_for_day(self, day, puuid):
        matches = self.matchlist_by_account(
            puuid,
            queue=420,
            begin_time=int(datetime.strptime(day, '%Y-%m-%d').timestamp()) * 1000,
            end_time=(int(datetime.strptime(day, '%Y-%m-%d').timestamp()) + 86400) * 1000)
        try:
            return matches['matches']
        except KeyError:
            return []

    def kills_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        return participant_info['kills']

    def deaths_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        return participant_info['deaths']

    def assists_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        return participant_info['assists']

    def kpp_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]

        if int(participant_id) <= 5:
            total_team_kills = sum([self.kills_for_participant(id, match_data) for id in range(1, 6)])
        else:
            total_team_kills = sum([self.kills_for_participant(id, match_data) for id in range(6, 11)])

        if not total_team_kills:
            return 0
        return ((participant_info['kills'] + participant_info['assists']) / total_team_kills) * 100

    def dmg_to_champs_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        return participant_info['totalDamageDealtToChampions']

    def tot_dmg_to_champs_per_min_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        return participant_info['totalDamageDealtToChampions'] / (match_data['gameDuration'] / 60)

    def gold_per_min_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        participant_stats = participant_info['stats']
        return participant_stats['goldEarned'] / (match_data['gameDuration'] / 60)

    def vision_score_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        return participant_info['visionScore']

    def kdr_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        deaths = participant_info['deaths']
        if not deaths: deaths = 1
        return (participant_info['kills'] + participant_info['assists']) / deaths

    def kda_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        return str(participant_info['kills']) + "/" + str(participant_info['deaths']) + "/" + str(
            participant_info['assists'])

    def win_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        return participant_info['win']

    def csdiff_at_10_for_participant(self, participant_id, match_data, timeline_data):
        minute_10_frames = timeline_data['frames'][10]['participantFrames']
        participant_info = \
            [info for key, info in minute_10_frames.items() if info['participantId'] == int(participant_id)][0]

        roles = roleml.predict(match_data, timeline_data)
        participant_role = roles[int(participant_id)]
        opponent_participant_id = \
            [id for id, role in roles.items() if role == participant_role and id != int(participant_id)][0]

        opponent_participant_info = \
            [info for key, info in minute_10_frames.items() if info['participantId'] == int(opponent_participant_id)][0]

        return participant_info['minionsKilled'] - opponent_participant_info['minionsKilled']

    def xpdiff_at_10_for_participant(self, participant_id, match_data, timeline_data):
        minute_10_frames = timeline_data['frames'][10]['participantFrames']
        participant_info = \
            [info for key, info in minute_10_frames.items() if info['participantId'] == int(participant_id)][0]

        roles = roleml.predict(match_data, timeline_data)
        participant_role = roles[int(participant_id)]
        opponent_participant_id = \
            [id for id, role in roles.items() if role == participant_role and id != int(participant_id)][0]

        opponent_participant_info = \
            [info for key, info in minute_10_frames.items() if info['participantId'] == int(opponent_participant_id)][0]

        return participant_info['xp'] - opponent_participant_info['xp']

    def control_wards_purchased_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        control_wards_purchased = participant_info['visionWardsBoughtInGame']
        return float(control_wards_purchased)

    def wards_placed_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        wards_placed = participant_info['wardsPlaced']
        return float(wards_placed)

    def wards_destroyed_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        wards_destroyed = participant_info['wardsKilled']
        return float(wards_destroyed)

    def control_wards_placed_for_participant(self, participant_id, timeline_data):
        frames = timeline_data['frames']

        wards = 0
        for frame in frames:
            if frame.get('events'):
                wards += len([event for event in frame['events']
                              if event['type'] == 'WARD_PLACED'
                              and event['creatorId'] == int(participant_id)
                              and event["wardType"] == "CONTROL_WARD"
                              ])

        return wards

    def solo_kills_for_participant(self, participant_id, timeline_data):
        frames = timeline_data['frames']
        count = 0
        for frame in frames:
            if frame.get('events'):
                solo_kills_maybe = [event for event in frame['events']
                                    if event['type'] == 'CHAMPION_KILL'
                                    and event['killerId'] == int(participant_id)
                                    and event['assistingParticipantIds'] == []
                                    ]
                # print(solo_kills_maybe)
                count += len(solo_kills_maybe)
        return count

    def solo_deaths_for_participant(self, participant_id, timeline_data):
        frames = timeline_data['frames']
        count = 0
        for frame in frames:
            if frame.get('events'):
                solo_deaths_maybe = [event for event in frame['events']
                                     if event['type'] == 'CHAMPION_KILL'
                                     and event['victimId'] == int(participant_id)
                                     and event['assistingParticipantIds'] == []
                                     ]
                # print(solo_kills_maybe)
                count += len(solo_deaths_maybe)
        return count

    def trinket_switch_time_for_participant(self, participant_id, timeline_data):
        frames = timeline_data['frames']
        timestamp = 0
        for frame in frames:
            if frame.get('events'):
                timestamp_maybe = [event for event in frame['events']
                                   if event['type'] == 'ITEM_PURCHASED'
                                   and (event['itemId'] == 3364 or event['itemId'] == 3187)
                                   and event["participantId"] == int(participant_id)
                                   ]
                if timestamp_maybe:
                    timestamp = timestamp_maybe[0]["timestamp"]
        return timestamp / 60000

    def death_total_for_participant_team(self, participant_id, match_data):
        if int(participant_id) <= 5:
            total_team_deaths = sum([info['deaths']
                                     for info in match_data['info']['participants'] if int(info['participantId']) <= 5])
        else:
            total_team_deaths = sum([info['deaths']
                                     for info in match_data['info']['participants'] if int(info['participantId']) >= 5])
        return total_team_deaths

    def kill_total_for_participant_team(self, participant_id, match_data):
        if int(participant_id) <= 5:
            total_team_kills = sum([info['kills']
                                    for info in match_data['info']['participants'] if int(info['participantId']) <= 5])
        else:
            total_team_kills = sum([info['kills']
                                    for info in match_data['info']['participants'] if int(info['participantId']) >= 5])
        return total_team_kills

    def dmg_total_for_participant_team(self, participant_id, match_data):
        if int(participant_id) <= 5:
            total_team_dmg = sum([info['totalDamageDealtToChampions']
                                  for info in match_data['info']['participants'] if int(info['participantId']) <= 5])
        else:
            total_team_dmg = sum([info['totalDamageDealtToChampions']
                                  for info in match_data['info']['participants'] if int(info['participantId']) >= 5])
        return total_team_dmg

    def dmg_percent_for_participant(self, participant_id, match_data):
        total_team_dmg = self.dmg_total_for_participant_team(participant_id, match_data)
        participant_dmg = self.dmg_to_champs_for_participant(participant_id, match_data)
        return (participant_dmg / total_team_dmg) * 100

    def cs_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        total_cs_killed = participant_info['totalMinionsKilled']
        return total_cs_killed

    def cs_per_min_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        total_cs_killed = participant_info['totalMinionsKilled']
        game_length_in_minutes = match_data['gameDuration'] / 60

        return total_cs_killed / game_length_in_minutes

    def champion_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        champion_id = participant_info['championId']
        champion_name = self.champion.by_id(champion_id).get('name')

        return champion_name

    def role_for_participant(self, participant_id, match_data):
        participant_info = [info for info in match_data['info']['participants'] if info['participantId'] == participant_id][0]
        participant_role = participant_info['individualPosition']
        return participant_role

    def lane_opponent_for_participant(self, participant_id, match_data, timeline_data):

        participant_role = self.role_for_participant(participant_id)
        opponent_participant_id = [info['participantId'] for info in match_data['info']['participants']
                                   if participant_role == info['individualPosition']
                                   and participant_id != info['participantId']]
        opponent_info = \
            [info for info in match_data['info']['participants'] if info['participantId'] == opponent_participant_id][0]
        champion_id = opponent_info['championId']
        champion_name = self.champion.by_id(champion_id).get('name')
        return champion_name

    def deaths_at_15_for_participant(self, participant_id, timeline_data):
        frames_of_interest = timeline_data['frames'][:16]

        deaths = 0
        for frame in frames_of_interest:
            if frame.get('events'):
                deaths += len([event for event in frame['events'] if
                               event['type'] == 'CHAMPION_KILL' and event['victimId'] == int(participant_id)])

        return deaths

    def csdiff_at_15_for_participant(self, participant_id, match_data, timeline_data):
        minute_15_frames = timeline_data['frames'][15]['participantFrames']
        participant_info = \
            [info for key, info in minute_15_frames.items() if info['participantId'] == int(participant_id)][0]

        roles = roleml.predict(match_data, timeline_data)
        participant_role = roles[int(participant_id)]
        opponent_participant_id = \
            [id for id, role in roles.items() if role == participant_role and id != int(participant_id)][0]

        opponent_participant_info = \
            [info for key, info in minute_15_frames.items() if info['participantId'] == int(opponent_participant_id)][0]

        return participant_info['minionsKilled'] - opponent_participant_info['minionsKilled']

    def xpdiff_at_15_for_participant(self, participant_id, match_data, timeline_data):
        minute_15_frames = timeline_data['frames'][15]['participantFrames']
        participant_info = \
            [info for key, info in minute_15_frames.items() if info['participantId'] == int(participant_id)][0]

        roles = roleml.predict(match_data, timeline_data)
        participant_role = roles[int(participant_id)]
        opponent_participant_id = \
            [id for id, role in roles.items() if role == participant_role and id != int(participant_id)][0]

        opponent_participant_info = \
            [info for key, info in minute_15_frames.items() if info['participantId'] == int(opponent_participant_id)][0]

        return participant_info['xp'] - opponent_participant_info['xp']

    def golddiff_at_10_for_participant(self, participant_id, match_data, timeline_data):
        minute_10_frames = timeline_data['frames'][15]['participantFrames']
        participant_info = \
            [info for key, info in minute_10_frames.items() if info['participantId'] == int(participant_id)][0]

        roles = roleml.predict(match_data, timeline_data)
        participant_role = roles[int(participant_id)]
        opponent_participant_id = \
            [id for id, role in roles.items() if role == participant_role and id != int(participant_id)][0]

        opponent_participant_info = \
            [info for key, info in minute_10_frames.items() if info['participantId'] == int(opponent_participant_id)][0]

        return participant_info['totalGold'] - opponent_participant_info['totalGold']

    def golddiff_at_15_for_participant(self, participant_id, match_data, timeline_data):
        minute_15_frames = timeline_data['frames'][15]['participantFrames']
        participant_info = \
            [info for key, info in minute_15_frames.items() if info['participantId'] == int(participant_id)][0]

        roles = roleml.predict(match_data, timeline_data)
        participant_role = roles[int(participant_id)]
        opponent_participant_id = \
            [id for id, role in roles.items() if role == participant_role and id != int(participant_id)][0]

        opponent_participant_info = \
            [info for key, info in minute_15_frames.items() if info['participantId'] == int(opponent_participant_id)][0]

        return participant_info['totalGold'] - opponent_participant_info['totalGold']

    def first_blood_for_side(self, side, match_data):
        side_info = [info for info in match_data['info']['participants']
                     if info['firstBloodKill']
                     and info['teamId'] == side][0]
        return side_info['firstBlood']

    def first_turret_for_side(self, side, match_data):
        side_info = [info['objectives'] for info in match_data['teams'] if info['teamId'] == side][0]
        return side_info['tower']['first']

    def heralds_for_side(self, side, match_data):
        side_info = [info['objectives'] for info in match_data['teams'] if info['teamId'] == side][0]
        return side_info['riftHerald']['kills']

    def barons_for_side(self, side, match_data):
        side_info = [info['objectives'] for info in match_data['teams'] if info['teamId'] == side][0]
        return side_info['baron']['kills']

    def dragons_for_side(self, side, match_data):
        side_info = [info['objectives'] for info in match_data['teams'] if info['teamId'] == side][0]
        return side_info['dragon']['kills']



import responses
import json
from blitzcrank import Blitzcrank


class TestLeagueEndpoint:

    def test_by_summoner_id(self):
        summoner_id = "fakefakefake"
        region = "euw1"
        expected = {"player": "info"}
        secret = "foo"

        b = Blitzcrank(secret, region)

        with responses.RequestsMock() as rsps:
            rsps.add(responses.GET,
                     f'https://{region}.api.riotgames.com/lol/league/v4/entries/by-summoner/{summoner_id}',
                     body=json.dumps(expected))

            actual = b.league.by_summoner_id(summoner_id)

            assert actual == expected
            assert rsps.calls[0].request.url == f'https://{region}.api.riotgames.com/lol/league/v4/entries/by-summoner/{summoner_id}'
            assert rsps.calls[0].request.headers['X-Riot-Token'] == secret
